#include "ogreapplication.h"
#include <OgreErrorDialog.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interface of MyApplication
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class MyApplication : public OgreApplication
{
public:
  MyApplication();
  virtual void keyClicked(Ogre::KeyEvent* e);
  bool frameEnded(const Ogre::FrameEvent& evt);
protected:
  virtual void init();
  Ogre::Overlay* mPanelOverlay;
};

using namespace Ogre;
using namespace std;

//************************************************************
// Implementation of MyApplication
//************************************************************
MyApplication::MyApplication()
  : OgreApplication("My Application")
{
}
void
MyApplication::keyClicked(Ogre::KeyEvent* e)
{
  if (e->getKey() == KC_F1)
  {
    if (mPanelOverlay->isVisible())
      mPanelOverlay->hide();
    else
      mPanelOverlay->show();      
    e->consume();
  }
  else OgreApplication::keyClicked(e);
}
void
MyApplication::init()
{
  mSceneManager->setAmbientLight(ColourValue(1,1,1));
      
  Entity *ent1 = mSceneManager->createEntity("Robot1","robot.mesh");
  SceneNode* node1 = mSceneManager->getRootSceneNode()->createChildSceneNode("Robot1Node");
  node1->attachObject(ent1);

  mPanelOverlay = OverlayManager::getSingleton().getByName("Tutorial2/PanelOverlay");
  mPanelOverlay->show();
}
bool
MyApplication::frameEnded(const FrameEvent& evt)
{
  static const String currFps  = "Current FPS: ";
  static const String avgFps   = "Average FPS: ";
  static const String bestFps  = "Best FPS: ";
  static const String worstFps = "Worst FPS: ";
  static const String tris     = "Triangle Count: ";

  if (mPanelOverlay && mPanelOverlay->isVisible())
  {
    OverlayElement* guiCurr  = OverlayManager::getSingleton().getOverlayElement("Tutorial2/CurrFps");
    OverlayElement* guiAvg   = OverlayManager::getSingleton().getOverlayElement("Tutorial2/AverageFps");
    OverlayElement* guiBest  = OverlayManager::getSingleton().getOverlayElement("Tutorial2/BestFps");
    OverlayElement* guiWorst = OverlayManager::getSingleton().getOverlayElement("Tutorial2/WorstFps");
    
    const RenderTarget::FrameStats& stats = mWindow->getStatistics();
    
    guiCurr->setCaption(currFps + StringConverter::toString(stats.lastFPS));
    guiAvg->setCaption(avgFps + StringConverter::toString(stats.avgFPS));
    guiBest->setCaption(bestFps + StringConverter::toString(stats.bestFPS)
			+" "+StringConverter::toString(stats.bestFrameTime)+" ms");
    guiWorst->setCaption(worstFps + StringConverter::toString(stats.worstFPS)
			 +" "+StringConverter::toString(stats.worstFrameTime)+" ms");
    
    OverlayElement* guiTris = OverlayManager::getSingleton().getOverlayElement("Tutorial2/NumTris");
    guiTris->setCaption(tris + StringConverter::toString(stats.triangleCount));  
  }
  
  return OgreApplication::frameEnded(evt);
}
int main(int, char **)
{
  MyApplication application;
  try
  {
    application.addResourceLocationFrom("../../resources.cfg");
    application.addResourceLocationFrom("./resources.cfg");
    application.create("../../plugins.cfg","../ogre.cfg","ogre.log",ST_GENERIC);
    application.run();
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

