#include "ogreapplication.h"
#include <OgreErrorDialog.h>

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Interface of MyApplication
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
class MyApplication : public OgreApplication
{
public:
  MyApplication();
  virtual void init();
};

using namespace Ogre;
using namespace std;

//************************************************************
// Implementation of MyApplication
//************************************************************
MyApplication::MyApplication()
  : OgreApplication("My Application")
{
}
void
MyApplication::init()
{
  mSceneManager->setAmbientLight(ColourValue(0.7f,0.7f,1.0f));
  
  // Adds a robot      
  Entity *robot = mSceneManager->createEntity("Robot","robot.mesh");
  SceneNode* robotNode = mSceneManager->getRootSceneNode()->createChildSceneNode("RobotNode");
  robotNode->attachObject(robot);
  Real r = robot->getBoundingRadius();
  
  // Adds a ground  
  MeshManager::getSingleton().createPlane("plane.mesh","Custom",Plane(Vector3::UNIT_Z,Vector3::ZERO),2*r,2*r,20,20);
  Entity* ground = mSceneManager->createEntity("Ground","plane.mesh");
  ground->setMaterialName("Examples/Rocky");
  SceneNode* groundNode = mSceneManager->getRootSceneNode()->createChildSceneNode();
  groundNode->attachObject(ground);
  groundNode->setOrientation(Quaternion(Radian(-Math::HALF_PI),Vector3::UNIT_X));
  
  // Adds a light 
  Light* light = mSceneManager->createLight("Light1"); 
  light->setType(Light::LT_SPOTLIGHT);
  light->setDiffuseColour(ColourValue(0.25f,0.25f,0.0f));
  light->setSpecularColour(ColourValue(0.25f,0.25f,0.0f));
  light->setAttenuation(8000,1,0.0005,0);
  light->setSpotlightRange(Ogre::Degree(60), Ogre::Degree(70));
  light->setDirection(Vector3::NEGATIVE_UNIT_Y);

  SceneNode* lightNode = robotNode->createChildSceneNode("LightNode");
  lightNode->attachObject(light);
  Real y = 1.2f*r;
  lightNode->setPosition(0,y,0);

  Entity* lightSphere = mSceneManager->createEntity("LightSphere","sphere.mesh");
  lightSphere->setMaterialName("Examples/Hilite/Yellow");
  lightNode->attachObject(lightSphere);
  Real s = 0.05f;
  lightNode->setScale(s,s,s);
}

int main(int, char **)
{
  MyApplication application;
  try
  {
    application.addResourceLocationFrom("../../resources.cfg");
    application.create("../../plugins.cfg","../ogre.cfg","ogre.log",ST_GENERIC);
    application.run();
  }
  catch (Exception& e)
  {
    ErrorDialog* dlg = PlatformManager::getSingleton().createErrorDialog();
    dlg->display(e.getFullDescription());
    delete dlg;
    return 1;
  }
  return 0;
}

